@@ -14,14 +14,14 @@ import (
 	"github.com/traefik/traefik/v2/pkg/provider/kubernetes/k8s"
 	traefikversion "github.com/traefik/traefik/v2/pkg/version"
 	corev1 "k8s.io/api/core/v1"
-	networkingv1 "k8s.io/api/networking/v1"
-	networkingv1beta1 "k8s.io/api/networking/v1beta1"
-	kubeerror "k8s.io/apimachinery/pkg/api/errors"
+	netv1 "k8s.io/api/networking/v1"
+	netv1beta1 "k8s.io/api/networking/v1beta1"
+	kerror "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/util/intstr"
-	"k8s.io/client-go/informers"
-	"k8s.io/client-go/kubernetes"
+	kinformers "k8s.io/client-go/informers"
+	kclientset "k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/clientcmd"
 )
@@ -31,30 +31,26 @@ const (
 	defaultTimeout = 5 * time.Second
 )
 
-type marshaler interface {
-	Marshal() ([]byte, error)
-}
-
 // Client is a client for the Provider master.
 // WatchAll starts the watch of the Provider resources and updates the stores.
 // The stores can then be accessed via the Get* functions.
 type Client interface {
 	WatchAll(namespaces []string, stopCh <-chan struct{}) (<-chan interface{}, error)
-	GetIngresses() []*networkingv1.Ingress
-	GetIngressClasses() ([]*networkingv1.IngressClass, error)
+	GetIngresses() []*netv1.Ingress
+	GetIngressClasses() ([]*netv1.IngressClass, error)
 	GetService(namespace, name string) (*corev1.Service, bool, error)
 	GetSecret(namespace, name string) (*corev1.Secret, bool, error)
 	GetEndpoints(namespace, name string) (*corev1.Endpoints, bool, error)
-	UpdateIngressStatus(ing *networkingv1.Ingress, ingStatus []corev1.LoadBalancerIngress) error
+	UpdateIngressStatus(ing *netv1.Ingress, ingStatus []netv1.IngressLoadBalancerIngress) error
 	GetServerVersion() *version.Version
 }
 
 type clientWrapper struct {
-	clientset            kubernetes.Interface
-	factoriesKube        map[string]informers.SharedInformerFactory
-	factoriesSecret      map[string]informers.SharedInformerFactory
-	factoriesIngress     map[string]informers.SharedInformerFactory
-	clusterFactory       informers.SharedInformerFactory
+	clientset            kclientset.Interface
+	factoriesKube        map[string]kinformers.SharedInformerFactory
+	factoriesSecret      map[string]kinformers.SharedInformerFactory
+	factoriesIngress     map[string]kinformers.SharedInformerFactory
+	clusterFactory       kinformers.SharedInformerFactory
 	ingressLabelSelector string
 	isNamespaceAll       bool
 	watchedNamespaces    []string
@@ -117,7 +113,7 @@ func createClientFromConfig(c *rest.Config) (*clientWrapper, error) {
 		runtime.GOARCH,
 	)
 
-	clientset, err := kubernetes.NewForConfig(c)
+	clientset, err := kclientset.NewForConfig(c)
 	if err != nil {
 		return nil, err
 	}
@@ -125,12 +121,12 @@ func createClientFromConfig(c *rest.Config) (*clientWrapper, error) {
 	return newClientImpl(clientset), nil
 }
 
-func newClientImpl(clientset kubernetes.Interface) *clientWrapper {
+func newClientImpl(clientset kclientset.Interface) *clientWrapper {
 	return &clientWrapper{
 		clientset:        clientset,
-		factoriesSecret:  make(map[string]informers.SharedInformerFactory),
-		factoriesIngress: make(map[string]informers.SharedInformerFactory),
-		factoriesKube:    make(map[string]informers.SharedInformerFactory),
+		factoriesSecret:  make(map[string]kinformers.SharedInformerFactory),
+		factoriesIngress: make(map[string]kinformers.SharedInformerFactory),
+		factoriesKube:    make(map[string]kinformers.SharedInformerFactory),
 	}
 }
 
@@ -168,23 +164,38 @@ func (c *clientWrapper) WatchAll(namespaces []string, stopCh <-chan struct{}) (<
 	}
 
 	for _, ns := range namespaces {
-		factoryIngress := informers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod, informers.WithNamespace(ns), informers.WithTweakListOptions(matchesLabelSelector))
+		factoryIngress := kinformers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod, kinformers.WithNamespace(ns), kinformers.WithTweakListOptions(matchesLabelSelector))
 
 		if supportsNetworkingV1Ingress(serverVersion) {
-			factoryIngress.Networking().V1().Ingresses().Informer().AddEventHandler(eventHandler)
+			_, err = factoryIngress.Networking().V1().Ingresses().Informer().AddEventHandler(eventHandler)
+			if err != nil {
+				return nil, err
+			}
 		} else {
-			factoryIngress.Networking().V1beta1().Ingresses().Informer().AddEventHandler(eventHandler)
+			_, err = factoryIngress.Networking().V1beta1().Ingresses().Informer().AddEventHandler(eventHandler)
+			if err != nil {
+				return nil, err
+			}
 		}
 
 		c.factoriesIngress[ns] = factoryIngress
 
-		factoryKube := informers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod, informers.WithNamespace(ns))
-		factoryKube.Core().V1().Services().Informer().AddEventHandler(eventHandler)
-		factoryKube.Core().V1().Endpoints().Informer().AddEventHandler(eventHandler)
+		factoryKube := kinformers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod, kinformers.WithNamespace(ns))
+		_, err = factoryKube.Core().V1().Services().Informer().AddEventHandler(eventHandler)
+		if err != nil {
+			return nil, err
+		}
+		_, err = factoryKube.Core().V1().Endpoints().Informer().AddEventHandler(eventHandler)
+		if err != nil {
+			return nil, err
+		}
 		c.factoriesKube[ns] = factoryKube
 
-		factorySecret := informers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod, informers.WithNamespace(ns), informers.WithTweakListOptions(notOwnedByHelm))
-		factorySecret.Core().V1().Secrets().Informer().AddEventHandler(eventHandler)
+		factorySecret := kinformers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod, kinformers.WithNamespace(ns), kinformers.WithTweakListOptions(notOwnedByHelm))
+		_, err = factorySecret.Core().V1().Secrets().Informer().AddEventHandler(eventHandler)
+		if err != nil {
+			return nil, err
+		}
 		c.factoriesSecret[ns] = factorySecret
 	}
 
@@ -215,12 +226,18 @@ func (c *clientWrapper) WatchAll(namespaces []string, stopCh <-chan struct{}) (<
 	}
 
 	if supportsIngressClass(serverVersion) {
-		c.clusterFactory = informers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod)
+		c.clusterFactory = kinformers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod)
 
 		if supportsNetworkingV1Ingress(serverVersion) {
-			c.clusterFactory.Networking().V1().IngressClasses().Informer().AddEventHandler(eventHandler)
+			_, err = c.clusterFactory.Networking().V1().IngressClasses().Informer().AddEventHandler(eventHandler)
+			if err != nil {
+				return nil, err
+			}
 		} else {
-			c.clusterFactory.Networking().V1beta1().IngressClasses().Informer().AddEventHandler(eventHandler)
+			_, err = c.clusterFactory.Networking().V1beta1().IngressClasses().Informer().AddEventHandler(eventHandler)
+			if err != nil {
+				return nil, err
+			}
 		}
 
 		c.clusterFactory.Start(stopCh)
@@ -236,8 +253,8 @@ func (c *clientWrapper) WatchAll(namespaces []string, stopCh <-chan struct{}) (<
 }
 
 // GetIngresses returns all Ingresses for observed namespaces in the cluster.
-func (c *clientWrapper) GetIngresses() []*networkingv1.Ingress {
-	var results []*networkingv1.Ingress
+func (c *clientWrapper) GetIngresses() []*netv1.Ingress {
+	var results []*netv1.Ingress
 
 	isNetworkingV1Supported := supportsNetworkingV1Ingress(c.serverVersion)
 
@@ -262,7 +279,7 @@ func (c *clientWrapper) GetIngresses() []*networkingv1.Ingress {
 		}
 
 		for _, ing := range list {
-			n, err := toNetworkingV1(ing)
+			n, err := convert[netv1.Ingress](ing)
 			if err != nil {
 				log.WithoutContext().Errorf("Failed to convert ingress %s from networking/v1beta1 to networking/v1: %v", ns, err)
 				continue
@@ -276,39 +293,9 @@ func (c *clientWrapper) GetIngresses() []*networkingv1.Ingress {
 	return results
 }
 
-func toNetworkingV1(ing marshaler) (*networkingv1.Ingress, error) {
-	data, err := ing.Marshal()
-	if err != nil {
-		return nil, err
-	}
-
-	ni := &networkingv1.Ingress{}
-	err = ni.Unmarshal(data)
-	if err != nil {
-		return nil, err
-	}
-
-	return ni, nil
-}
-
-func toNetworkingV1IngressClass(ing marshaler) (*networkingv1.IngressClass, error) {
-	data, err := ing.Marshal()
-	if err != nil {
-		return nil, err
-	}
-
-	ni := &networkingv1.IngressClass{}
-	err = ni.Unmarshal(data)
-	if err != nil {
-		return nil, err
-	}
-
-	return ni, nil
-}
-
-func addServiceFromV1Beta1(ing *networkingv1.Ingress, old networkingv1beta1.Ingress) {
+func addServiceFromV1Beta1(ing *netv1.Ingress, old netv1beta1.Ingress) {
 	if old.Spec.Backend != nil {
-		port := networkingv1.ServiceBackendPort{}
+		port := netv1.ServiceBackendPort{}
 		if old.Spec.Backend.ServicePort.Type == intstr.Int {
 			port.Number = old.Spec.Backend.ServicePort.IntVal
 		} else {
@@ -316,8 +303,8 @@ func addServiceFromV1Beta1(ing *networkingv1.Ingress, old networkingv1beta1.Ingr
 		}
 
 		if old.Spec.Backend.ServiceName != "" {
-			ing.Spec.DefaultBackend = &networkingv1.IngressBackend{
-				Service: &networkingv1.IngressServiceBackend{
+			ing.Spec.DefaultBackend = &netv1.IngressBackend{
+				Service: &netv1.IngressServiceBackend{
 					Name: old.Spec.Backend.ServiceName,
 					Port: port,
 				},
@@ -333,14 +320,14 @@ func addServiceFromV1Beta1(ing *networkingv1.Ingress, old networkingv1beta1.Ingr
 			if path.Backend.Service == nil {
 				oldBackend := old.Spec.Rules[rc].HTTP.Paths[pc].Backend
 
-				port := networkingv1.ServiceBackendPort{}
+				port := netv1.ServiceBackendPort{}
 				if oldBackend.ServicePort.Type == intstr.Int {
 					port.Number = oldBackend.ServicePort.IntVal
 				} else {
 					port.Name = oldBackend.ServicePort.StrVal
 				}
 
-				svc := networkingv1.IngressServiceBackend{
+				svc := netv1.IngressServiceBackend{
 					Name: oldBackend.ServiceName,
 					Port: port,
 				}
@@ -352,7 +339,7 @@ func addServiceFromV1Beta1(ing *networkingv1.Ingress, old networkingv1beta1.Ingr
 }
 
 // UpdateIngressStatus updates an Ingress with a provided status.
-func (c *clientWrapper) UpdateIngressStatus(src *networkingv1.Ingress, ingStatus []corev1.LoadBalancerIngress) error {
+func (c *clientWrapper) UpdateIngressStatus(src *netv1.Ingress, ingStatus []netv1.IngressLoadBalancerIngress) error {
 	if !c.isWatchedNamespace(src.Namespace) {
 		return fmt.Errorf("failed to get ingress %s/%s: namespace is not within watched namespaces", src.Namespace, src.Name)
 	}
@@ -374,7 +361,7 @@ func (c *clientWrapper) UpdateIngressStatus(src *networkingv1.Ingress, ingStatus
 	}
 
 	ingCopy := ing.DeepCopy()
-	ingCopy.Status = networkingv1.IngressStatus{LoadBalancer: corev1.LoadBalancerStatus{Ingress: ingStatus}}
+	ingCopy.Status = netv1.IngressStatus{LoadBalancer: netv1.IngressLoadBalancerStatus{Ingress: ingStatus}}
 
 	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
 	defer cancel()
@@ -388,7 +375,7 @@ func (c *clientWrapper) UpdateIngressStatus(src *networkingv1.Ingress, ingStatus
 	return nil
 }
 
-func (c *clientWrapper) updateIngressStatusOld(src *networkingv1.Ingress, ingStatus []corev1.LoadBalancerIngress) error {
+func (c *clientWrapper) updateIngressStatusOld(src *netv1.Ingress, ingStatus []netv1.IngressLoadBalancerIngress) error {
 	ing, err := c.factoriesIngress[c.lookupNamespace(src.Namespace)].Networking().V1beta1().Ingresses().Lister().Ingresses(src.Namespace).Get(src.Name)
 	if err != nil {
 		return fmt.Errorf("failed to get ingress %s/%s: %w", src.Namespace, src.Name, err)
@@ -396,13 +383,23 @@ func (c *clientWrapper) updateIngressStatusOld(src *networkingv1.Ingress, ingSta
 
 	logger := log.WithoutContext().WithField("namespace", ing.Namespace).WithField("ingress", ing.Name)
 
-	if isLoadBalancerIngressEquals(ing.Status.LoadBalancer.Ingress, ingStatus) {
+	ingresses, err := convertSlice[netv1.IngressLoadBalancerIngress](ing.Status.LoadBalancer.Ingress)
+	if err != nil {
+		return err
+	}
+
+	if isLoadBalancerIngressEquals(ingresses, ingStatus) {
 		logger.Debug("Skipping ingress status update")
 		return nil
 	}
 
+	ingressesBeta1, err := convertSlice[netv1beta1.IngressLoadBalancerIngress](ingStatus)
+	if err != nil {
+		return err
+	}
+
 	ingCopy := ing.DeepCopy()
-	ingCopy.Status = networkingv1beta1.IngressStatus{LoadBalancer: corev1.LoadBalancerStatus{Ingress: ingStatus}}
+	ingCopy.Status = netv1beta1.IngressStatus{LoadBalancer: netv1beta1.IngressLoadBalancerStatus{Ingress: ingressesBeta1}}
 
 	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
 	defer cancel()
@@ -416,7 +413,7 @@ func (c *clientWrapper) updateIngressStatusOld(src *networkingv1.Ingress, ingSta
 }
 
 // isLoadBalancerIngressEquals returns true if the given slices are equal, false otherwise.
-func isLoadBalancerIngressEquals(aSlice, bSlice []corev1.LoadBalancerIngress) bool {
+func isLoadBalancerIngressEquals(aSlice, bSlice []netv1.IngressLoadBalancerIngress) bool {
 	if len(aSlice) != len(bSlice) {
 		return false
 	}
@@ -468,12 +465,12 @@ func (c *clientWrapper) GetSecret(namespace, name string) (*corev1.Secret, bool,
 	return secret, exist, err
 }
 
-func (c *clientWrapper) GetIngressClasses() ([]*networkingv1.IngressClass, error) {
+func (c *clientWrapper) GetIngressClasses() ([]*netv1.IngressClass, error) {
 	if c.clusterFactory == nil {
 		return nil, errors.New("cluster factory not loaded")
 	}
 
-	var ics []*networkingv1.IngressClass
+	var ics []*netv1.IngressClass
 	if !supportsNetworkingV1Ingress(c.serverVersion) {
 		ingressClasses, err := c.clusterFactory.Networking().V1beta1().IngressClasses().Lister().List(labels.Everything())
 		if err != nil {
@@ -482,7 +479,7 @@ func (c *clientWrapper) GetIngressClasses() ([]*networkingv1.IngressClass, error
 
 		for _, ic := range ingressClasses {
 			if ic.Spec.Controller == traefikDefaultIngressClassController {
-				icN, err := toNetworkingV1IngressClass(ic)
+				icN, err := convert[netv1.IngressClass](ic)
 				if err != nil {
 					log.WithoutContext().Errorf("Failed to convert ingress class %s from networking/v1beta1 to networking/v1: %v", ic.Name, err)
 					continue
@@ -529,7 +526,7 @@ func (c *clientWrapper) GetServerVersion() *version.Version {
 // translateNotFoundError will translate a "not found" error to a boolean return
 // value which indicates if the resource exists and a nil error.
 func translateNotFoundError(err error) (bool, error) {
-	if kubeerror.IsNotFound(err) {
+	if kerror.IsNotFound(err) {
 		return false, nil
 	}
 	return err == nil, err
@@ -558,8 +555,8 @@ func supportsIngressClass(serverVersion *version.Version) bool {
 }
 
 // filterIngressClassByName return a slice containing ingressclasses with the correct name.
-func filterIngressClassByName(ingressClassName string, ics []*networkingv1.IngressClass) []*networkingv1.IngressClass {
-	var ingressClasses []*networkingv1.IngressClass
+func filterIngressClassByName(ingressClassName string, ics []*netv1.IngressClass) []*netv1.IngressClass {
+	var ingressClasses []*netv1.IngressClass
 
 	for _, ic := range ics {
 		if ic.Name == ingressClassName {
